import { createSignal } from "solid-js";
import { localState } from "./state";

export interface User {
    username: string,
    displayName: string,
    googleId: string,
}

export interface AuthenticationResponse {
    user: User,
    token: string,
}

export interface AuthConfig {
    clientId: string,
    redirect_url: string,
}

export interface Group {
    id: string,
    name: string,
    owner: User,
    members: User[],
    strats: Strat[],
}

export interface Strat {
    id: string,
    title: string,
    description: string,
    stratType: string,
    attempts: number,
    success: number
    stratStates: StratState[],
    playerTypes: PlayerType[],
    map: Map,
}

export interface Map {
    id: string,
    name: string,
    image: File,
}

export interface StratState {
    id: string,
    description: string,
    image: File,
    lineups: Lineup[],
}

export interface Lineup {
    id: string,
    description: string,
    image: File,
}

export interface PlayerType {
    id: string,
    name: string,
    task: string,
    profiles: Profile[],
}

export interface Profile {
    id: string,
    name: string,
    image: File,
}

export interface GroupRequest {
    name: string,
}

export interface StratRequest {
    title: string,
    description: string,
    stratType: string,
    attempts: number,
    success: number,
}

export class API {
    public apiURL: string;

    constructor(apiURL: string) {
        this.apiURL = apiURL;
    }

    private async checkResponse(response: Response) {
        if (!response.ok) {
            let json = null;
            try {
                json = await response.json();
            } catch (e) { /* ignored */ }

            throw json?.error || 'Request failed: ' + response.status + ' ' + response.statusText;
        }
    }

    public ensureLoggedIn(): AuthenticationResponse {
        const info = localState.accountInfo;
        if (info == null) throw 'Not logged in';
        return info;
    }

    public async authConfig(): Promise<AuthConfig> {
        const response = await fetch(this.apiURL + '/config');

        await this.checkResponse(response);

        return await response.json();
    }

    public async login(code: string): Promise<AuthenticationResponse> {
        const response = await fetch(this.apiURL + '/auth/login', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify({ code })
        });

        await this.checkResponse(response);

        return await response.json();
    }

    public async getGroups(): Promise<Group[]> {
        const response = await fetch(this.apiURL + '/group', {
            method: 'GET',
            headers: {
                'Authorization': 'Bearer ' + this.ensureLoggedIn().token
            }
        });

        await this.checkResponse(response);

        return await response.json();
    }

    public async createGroup(group: GroupRequest): Promise<Group> {
        const response = await fetch(this.apiURL + '/group', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Authorization': 'Bearer ' + this.ensureLoggedIn().token
            },
            body: JSON.stringify(group)
        });

        await this.checkResponse(response);

        return await response.json();
    }

    public async updateGroup(id: string, group: GroupRequest): Promise<Group> {
        const response = await fetch(this.apiURL + '/group/' + id, {
            method: 'PUT',
            headers: {
                'Content-Type': 'application/json',
                'Authorization': 'Bearer ' + this.ensureLoggedIn().token
            },
            body: JSON.stringify(group)
        });

        await this.checkResponse(response);

        return await response.json();
    }

    public async getGroup(id: string): Promise<Group> {
        const response = await fetch(this.apiURL + '/group/' + id, {
            method: 'GET',
            headers: {
                'Authorization': 'Bearer ' + this.ensureLoggedIn().token
            }
        });

        await this.checkResponse(response);

        return await response.json();
    }

    public async addMember(groupId: string, memberId: string): Promise<Group> {
        const response = await fetch(this.apiURL + '/group/' + groupId + '/member', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Authorization': 'Bearer ' + this.ensureLoggedIn().token
            },
            body: JSON.stringify({ userId: memberId })
        });

        await this.checkResponse(response);

        return await response.json();
    }

    public async removeMember(groupId: string, memberId: string): Promise<Group> {
        const response = await fetch(this.apiURL + '/group/' + groupId + '/member/' + memberId, {
            method: 'DELETE',
            headers: {
                'Content-Type': 'application/json',
                'Authorization': 'Bearer ' + this.ensureLoggedIn().token
            },
        });

        await this.checkResponse(response);

        return await response.json();
    }

    public async addStrat(groupId: string, strat: StratRequest): Promise<Strat> {
        const response = await fetch(this.apiURL + '/group/' + groupId + '/strat', {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Authorization': 'Bearer ' + this.ensureLoggedIn().token
            },
            body: JSON.stringify(strat)
        });

        await this.checkResponse(response);

        return await response.json();
    }

    public async removeStrat(groupId: string, stratId: string): Promise<void> {
        const response = await fetch(this.apiURL + '/group/' + groupId + '/strat/' + stratId, {
            method: 'DELETE',
            headers: {
                'Content-Type': 'application/json',
                'Authorization': 'Bearer ' + this.ensureLoggedIn().token
            }
        });

        await this.checkResponse(response);
    }

    public async updateStrat(groupId: string, stratId: string, strat: StratRequest): Promise<Strat> {
        const response = await fetch(this.apiURL + '/group/' + groupId + '/strat/' + stratId, {
            method: 'PUT',
            headers: {
                'Content-Type': 'application/json',
                'Authorization': 'Bearer ' + this.ensureLoggedIn().token
            },
            body: JSON.stringify(strat)
        });

        await this.checkResponse(response);

        return await response.json();
    }
}

export const [api, setAPI] = createSignal(new API(''));
export const [authConfig, setAuthConfig] = createSignal(null as AuthConfig | null)
