import { useNavigate, useParams } from "@solidjs/router";
import { Component, Show, createEffect, createSignal } from "solid-js";

import styles from './GroupPage.module.css'
import { Group, User, api } from "../api";
import { showDialog, showInputDialog, showMessageDialog } from "../state";
import { errorToString, normalize } from "../util";
import { BsCheck2, BsPencil, BsTrash } from "solid-icons/bs";

const GroupPage: Component = () => {
	const params = useParams();
	const navigate = useNavigate();

	const [loading, setLoading] = createSignal(true);
	const [group, setGroup] = createSignal({} as Group);
	const [editing, setEditing] = createSignal(false);
	const [name, setName] = createSignal('');

	createEffect(async () => {
		try {
			setGroup(await api().getGroup(params.id));
			setName(group().name);
			setLoading(false);
		} catch (e) {
			showMessageDialog('Failed to load group', errorToString(e));
		}
	}, []);

	const updateGroup = async (newValue: Group) => {
		try {
			await api().updateGroup(group().id, newValue);
			setGroup(newValue);
		} catch (e) {
			showMessageDialog('Failed to update group', errorToString(e));
		}
	};

	const deleteGroup = async () => {
		showDialog({
			title: 'Delete group',
			text: 'Do you want to delete this group?',
			buttons: [
				{
					name: 'Delete',
					type: 'danger',
					action: async () => {
						try {
							await api().deleteGroup(group().id);
							navigate('/groups');
						} catch (e) {
							showMessageDialog('Failed to delete group', errorToString(e));
						}
					}
				},
				{
					name: 'Cancel',
					action: () => { }
				}
			]
		});
	};

	const addMember = async () => {
		showInputDialog('Add member to Group', 'enter the member id you want to add', 'member ID', async (memberId) => {
			try {
				const newGroup = await api().addMember(group().id, memberId);
				setGroup(newGroup);
			} catch (e) {
				showMessageDialog('Failed to add member', errorToString(e));
			}
		})

	};

	const addWishlist = async () => {
		const userWishlists = await api().getWishlists(api().ensureLoggedIn().username);
		showDialog({
			title: 'Add Wishlist', text: 'add wishlist to group', buttons:
				[...userWishlists.map(w => {
					return {
						name: w.title, action: async () => {
							try {
								const newGroup = await api().addWishlistToGroup(group().id, w.id);
								setGroup(newGroup);
							} catch (e) {
								showMessageDialog('Failed to add wishlist', errorToString(e));
							}
						}
					}
				}),
					'cancel'
				]
		})
	};

	const removeWishlist = async (wishlistId: string) => {
		showDialog({
			title: 'Remove Wishlist', text: 'remove wishlist from group', buttons:
				[{
					name: 'remove', action: async () => {
						const newGroup = await api().removeWishlistFromGroup(group().id, wishlistId);
						setGroup(newGroup);
					}
				},
					'cancel'
				]
		})
	};

	const toggleEdit = async () => {
		if (!editing()) {
			setEditing(true);
		} else {
			console.log(name());
			updateGroup({ ...group(), name: name() });
			setEditing(false);
		}
	};

	const onDeleteMember = async (member: User) => {
		setGroup({ ...group(), members: [...group().members.filter(m => m.username !== member.username)] });
	};

	return (
		<>
			<Show when={!loading()} fallback={<div>Loading...</div>}>
				<div class={styles.groupName}>
					<span contentEditable={editing()} innerText={group().name} onInput={e => setName(normalize(e.currentTarget.innerText, 50, true))} />
					{editing() && <span class={styles.editCount}>{name().length}/50</span>}
				</div>
				<div class={styles.groupButtons}>
					<button class={styles.wishlistButton} onClick={deleteGroup}>Delete group</button>
					<button class={styles.editButton} onClick={toggleEdit}>{editing() ? <BsCheck2 /> : <BsPencil />}</button>
				</div>
				<hr />
				<button class={styles.groupButton} onClick={addMember}>Add Member</button>
				<div class={styles.groupMembers}>
					{group().members?.map(m => <MemberItem group={group()} member={m} onDelete={onDeleteMember} />)}
				</div>
				<button class={styles.wishlistButton} onClick={addWishlist}>Add Wishlist</button>
				<div class={styles.groupWishlists}>
					{group().wishlists?.map(w => <div>
						<WishlistItem wishlist={w} />
						<button class={styles.removeWishlistButton} onClick={() => removeWishlist(w.id)}><BsTrash /></button>
					</div>)}
				</div>
			</Show>
		</>
	)
}

export default GroupPage;
